<?php
namespace Ep\Http\Validator;

use Psr\Http\Message\ResponseInterface;
use Zend\InputFilter\InputFilter;
use Zend\InputFilter\InputFilterInterface;
use Zend\Validator\AbstractValidator;

/**
 * Validira odgovor kao JSON.
 *
 * @package Ep\Http\Validator
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class JsonResponse extends AbstractValidator
{
    const NOT_JSON     = 'json_response.not_json';
    const INVALID_JSON = 'json_response.invalid_json';

    /** @var array */
    protected $messageTemplates = [
        self::NOT_JSON     => 'Dati odgovor nije vrste `application/json`.',
        self::INVALID_JSON => 'Dati odgovor ne odgovara zadanoj schemi. %value%',
    ];

    /** @var array */
    protected $options = [
        'schema' => '',
    ];

    /** @var InputFilter */
    protected $inputFilter;

    /**
     * @return InputFilter
     */
    public function getInputFilter(): InputFilter
    {
        return $this->inputFilter;
    }

    /**
     * @param InputFilterInterface $inputFilter
     */
    public function setInputFilter(InputFilterInterface $inputFilter): void
    {
        $this->inputFilter = $inputFilter;
    }

    /**
     * Provjerava da li odgovor odgovara zadanoj JSON strukturi.
     *
     * @param ResponseInterface $value
     *
     * @return bool
     */
    public function isValid($value)
    {
        if (!$value instanceof ResponseInterface) {
            throw new \InvalidArgumentException('Ovaj validator može validirati samo HTTP odgovore.');
        }

        if ($value->getHeaderLine('Content-Type') !== 'application/json') {
            $this->error(self::NOT_JSON);
            return false;
        }

        if ($this->inputFilter instanceof InputFilterInterface) {
            $data = json_decode($value->getBody()->getContents(), true);
            $this->inputFilter->setData($data);

            if (!$this->inputFilter->isValid()) {
                $errors = [];

                foreach ($this->inputFilter->getMessages() as $name => $messages) {
                    $errors[] = $name . ': ' . implode(' ', $messages);
                }

                $this->error(self::INVALID_JSON, "\n" . implode("\n", $errors));
                return false;
            }
        }

        return true;
    }
}